import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { CollectionPageService } from './collection-page-cms.service';
import { BaseCollectionPageDto } from './dto/base-collection-page-cms.dto';
import { UpdateCollectionPageDto } from './dto/update-collection-page-cms.dto';

@Controller({ path: 'collection-page', version: Constants.API_VERSION_1 })

export class CollectionPageController {
    constructor(private readonly service: CollectionPageService) { }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'blockOneImage', maxCount: 1 },
            { name: 'blockTwoImage', maxCount: 1 }
        ]
    ))
    create(
        @Body() dto: BaseCollectionPageDto,
        @UploadedFiles() files: {
            blockOneImage: UploadedMulterFileI,
            blockTwoImage: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'blockOneImage', maxCount: 1 },
            { name: 'blockTwoImage', maxCount: 1 }
        ]
    ))
    update(
        @Param('id') id: string,
        @Body() dto: UpdateCollectionPageDto,
        @UploadedFiles() files: {
            blockOneImage: UploadedMulterFileI,
            blockTwoImage: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
