import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { BaseCollectionPageDto } from './dto/base-collection-page-cms.dto';
import { CollectionPage } from './entities/collection-page-cms.entity';
import { UpdateCollectionPageDto } from './dto/update-collection-page-cms.dto';
import { CollectionPageRepository } from './collection-page-cms.repository';

@Injectable()
export class CollectionPageService {
    constructor(
        @InjectRepository(CollectionPageRepository)
        private readonly repository: CollectionPageRepository,
        private readonly spaceService: DoSpacesService
    ) { }

    async create(
        dto: BaseCollectionPageDto,
        files: { 
            blockOneImage: UploadedMulterFileI,
            blockTwoImage: UploadedMulterFileI
        }
    ): Promise<CollectionPage | Error> {
        if (files && files.blockOneImage) {
            const blockOneImage: any = await this.spaceService.uploadFile(files.blockOneImage[0], 'CollectionPage/');
            dto.blockOneImage = blockOneImage;
        }
        if (files && files.blockTwoImage) {
            const blockTwoImage: any = await this.spaceService.uploadFile(files.blockTwoImage[0], 'CollectionPage/');
            dto.blockTwoImage = blockTwoImage;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateCollectionPageDto,
        files: { 
            blockOneImage: UploadedMulterFileI,
            blockTwoImage: UploadedMulterFileI
        }
    ): Promise<CollectionPage | Error> {
        const res : any = await this.repository.findOneEntity(id);
        if(!res){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.blockOneImage) {
            const blockOneImage: any = await this.spaceService.uploadFile(files.blockOneImage[0], 'CollectionPage/');
            dto.blockOneImage = blockOneImage;
        }
        if (files && files.blockTwoImage) {
            const blockTwoImage: any = await this.spaceService.uploadFile(files.blockTwoImage[0], 'CollectionPage/');
            dto.blockTwoImage = blockTwoImage;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<CollectionPage[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<CollectionPage[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
