import { Controller, Get, Post, Body, HttpCode, UseGuards, HttpStatus, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { GiftGuideCmsService } from './gift-guide-cms.service';
import { UpdateGiftGuideCmsDto } from './dto/update-gift-guide-cms.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Controller({ path: 'gift-guide-cms', version: Constants.API_VERSION_1 })

export class GiftGuideCmsController {
    constructor(private readonly service: GiftGuideCmsService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'image', maxCount: 1 }
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateGiftGuideCmsDto,
        @UploadedFiles() files: { 
            image: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
