import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { LearnPageCmsRepository } from './learn-page-cms.repository';
import { UpdateLearnPageCmsDto } from './dto/update-learn-page-cms.dto';
import { LearnPageCms } from './entities/learn-page-cms.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class LearnPageCmsService {
    constructor(
        @InjectRepository(LearnPageCmsRepository)
        private readonly repository: LearnPageCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: UpdateLearnPageCmsDto,
        files: {
            sectionTwoImage: UploadedMulterFileI,
            sectionFourImage: UploadedMulterFileI,
            sectionSixImage: UploadedMulterFileI,
            sectionSevenImage: UploadedMulterFileI,
            sectionOneImage: UploadedMulterFileI
        }
    ): Promise<LearnPageCms | Error> {
        if (files && files.sectionOneImage) {
            const sectionOneImage: any = await this.doSpaceService.uploadFile(files.sectionOneImage[0], 'learnCMS/');
            dto.sectionOneImage = sectionOneImage;
        }
        if (files && files.sectionTwoImage) {
            const sectionTwoImage: any = await this.doSpaceService.uploadFile(files.sectionTwoImage[0], 'learnCMS/');
            dto.sectionTwoImage = sectionTwoImage;
        }
        if (files && files.sectionFourImage) {
            const sectionFourImage: any = await this.doSpaceService.uploadFile(files.sectionFourImage[0], 'learnCMS/');
            dto.sectionFourImage = sectionFourImage;
        }
        if (files && files.sectionSixImage) {
            const sectionSixImage: any = await this.doSpaceService.uploadFile(files.sectionSixImage[0], 'learnCMS/');
            dto.sectionSixImage = sectionSixImage;
        }
        if (files && files.sectionSevenImage) {
            const sectionSevenImage: any = await this.doSpaceService.uploadFile(files.sectionSevenImage[0], 'learnCMS/');
            dto.sectionSevenImage = sectionSevenImage;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if(Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if(!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if(!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<LearnPageCms[] | Error> {
        const data = await this.repository.findAll();
        if(!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
