import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductPageCmsRepository } from './product-page-cms.repository';
import { UpdateProductPageCmsDto } from './dto/update-product-page-cms.dto';
import { ProductPageCms } from './entities/product-page-cms.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class ProductPageCmsService {
    constructor(
        @InjectRepository(ProductPageCmsRepository)
        private readonly repository: ProductPageCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: UpdateProductPageCmsDto,
        files: {
            discountImage: UploadedMulterFileI,
        }
    ): Promise<ProductPageCms | Error> {
        // process image one
        if (files && files.discountImage) {
            const discountImage: any = await this.doSpaceService.uploadFile(files.discountImage[0], 'ProductPageCms/');
            dto.discountImage = discountImage;
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<ProductPageCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product page cms data fetched successfully", "data", data);
    }
}
