import { Controller, Get, Post, Body, HttpCode, UseGuards, HttpStatus, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { getMulterConfig } from 'src/core/multer.config';
import { Constants } from 'src/utils/constants';
import { PromotionalPageCmsService } from './promotional-page-cms.service';
import { UpdatePromotionalPageCmsDto } from './dto/update-promotional-page-cms.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Controller({ path: 'promotional-page-cms', version: Constants.API_VERSION_1 })

export class PromotionalPageCmsController {
    constructor(private readonly service: PromotionalPageCmsService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'firstProductImage', maxCount: 1 },
            { name: 'secondProductImage', maxCount: 1 },
            { name: 'thirdProductImage', maxCount: 1 },
            { name: 'fourthProductImage', maxCount: 1 },
            { name: 'fifthProductImage', maxCount: 1 },
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdatePromotionalPageCmsDto,
        @UploadedFiles() files: { 
            firstProductImage: UploadedMulterFileI,
            secondProductImage: UploadedMulterFileI,
            thirdProductImage: UploadedMulterFileI,
            fourthProductImage: UploadedMulterFileI,
            fifthProductImage: UploadedMulterFileI,
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
