import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { PromotionalPageCmsRepository } from './promotional-page-cms.repository';
import { UpdatePromotionalPageCmsDto } from './dto/update-promotional-page-cms.dto';
import { PromotionalPageCms } from './entities/promotional-page-cms.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { ProductsRepository } from 'src/modules/product/product/product.repository';

@Injectable()
export class PromotionalPageCmsService {
    constructor(
        @InjectRepository(PromotionalPageCmsRepository)
        private readonly repository: PromotionalPageCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService,
        @InjectRepository(ProductsRepository)
        private readonly productRepository: ProductsRepository,
    ) { }

    async updateOrCreateData(
        dto: UpdatePromotionalPageCmsDto,
        files: {
            firstProductImage: UploadedMulterFileI,
            secondProductImage: UploadedMulterFileI,
            thirdProductImage: UploadedMulterFileI,
            fourthProductImage: UploadedMulterFileI,
            fifthProductImage: UploadedMulterFileI,
        }
    ): Promise<PromotionalPageCms | Error> {
        if (files && files.firstProductImage) {
            const firstProductImage: any = await this.doSpaceService.uploadFile(files.firstProductImage[0], 'PromotionalCMS/');
            dto.firstProductImage = firstProductImage;
        }
        if (files && files.secondProductImage) {
            const secondProductImage: any = await this.doSpaceService.uploadFile(files.secondProductImage[0], 'PromotionalCMS/');
            dto.secondProductImage = secondProductImage;
        }
        if (files && files.thirdProductImage) {
            const thirdProductImage: any = await this.doSpaceService.uploadFile(files.thirdProductImage[0], 'PromotionalCMS/');
            dto.thirdProductImage = thirdProductImage;
        }
        if (files && files.fourthProductImage) {
            const fourthProductImage: any = await this.doSpaceService.uploadFile(files.fourthProductImage[0], 'PromotionalCMS/');
            dto.fourthProductImage = fourthProductImage;
        }

        if (files && files.fifthProductImage) {
            const fifthProductImage: any = await this.doSpaceService.uploadFile(files.fifthProductImage[0], 'PromotionalCMS/');
            dto.fifthProductImage = fifthProductImage;
        }

        // work with discount amount
        const firstProduct = await this.productRepository.findOne(dto.firstProductId);
        if (!firstProduct) {
            throw new HttpException(
                "First product not found",
                HttpStatus.NOT_FOUND
            );
        }
        else{
            const productPrice = firstProduct.price;
            if (dto.firstProductDiscount.includes('%')) {
                const firstProductDiscount = dto.firstProductDiscount.replace('%', '');
                const discount: any = (productPrice * (parseInt(firstProductDiscount) / 100)).toFixed(2);
                const discountPrice: any = (productPrice - discount).toFixed(2);
                await this.productRepository.update(dto.firstProductId, { discountPrice: discountPrice });        
            }
            else {
                const discountPrice: any = (productPrice - parseInt(dto.firstProductDiscount)).toFixed(2);
                await this.productRepository.update(dto.firstProductId, { discountPrice: discountPrice });
            }
        }
        
        const secondProduct = await this.productRepository.findOne(dto.secondProductId);
        if (!secondProduct) {
            throw new HttpException(
                "Second product not found",
                HttpStatus.NOT_FOUND
            );
        }
        else{
            const productPrice = secondProduct.price;
            if (dto.secondProductDiscount.includes('%')) {
                const secondProductDiscount = dto.secondProductDiscount.replace('%', '');
                const discount: any = (productPrice * (parseInt(secondProductDiscount) / 100)).toFixed(2);
                const discountPrice: any = (productPrice - discount).toFixed(2);
                await this.productRepository.update(dto.secondProductId, { discountPrice: discountPrice });
            }
            else {
                const discountPrice: any = (productPrice - parseInt(dto.secondProductDiscount)).toFixed(2);
                await this.productRepository.update(dto.secondProductId, { discountPrice: discountPrice });
            }
        }

        const thirdProduct = await this.productRepository.findOne(dto.thirdProductId);
        if (!thirdProduct) {
            throw new HttpException(
                "Third product not found",
                HttpStatus.NOT_FOUND
            );
        }
        else{
            const productPrice = thirdProduct.price;
            if (dto.thirdProductDiscount.includes('%')) {
                const thirdProductDiscount = dto.thirdProductDiscount.replace('%', '');
                const discount: any = (productPrice * (parseInt(thirdProductDiscount) / 100)).toFixed(2);
                const discountPrice: any = (productPrice - discount).toFixed(2);
                await this.productRepository.update(dto.thirdProductId, { discountPrice: discountPrice });
            }
            else {
                const discountPrice: any = (productPrice - parseInt(dto.thirdProductDiscount)).toFixed(2);
                await this.productRepository.update(dto.thirdProductId, { discountPrice: discountPrice });
            }
        }

        const fourthProduct = await this.productRepository.findOne(dto.fourthProductId);
        if (!fourthProduct) {
            throw new HttpException(
                "Fourth product not found",
                HttpStatus.NOT_FOUND
            );
        }
        else{
            const productPrice = fourthProduct.price;
            if (dto.fourthProductDiscount.includes('%')) {
                const fourthProductDiscount = dto.fourthProductDiscount.replace('%', '');
                const discount: any = (productPrice * (parseInt(fourthProductDiscount) / 100)).toFixed(2);
                const discountPrice: any = (productPrice - discount).toFixed(2);
                await this.productRepository.update(dto.fourthProductId, { discountPrice: discountPrice });
            }
            else {
                const discountPrice: any = (productPrice - parseInt(dto.fourthProductDiscount)).toFixed(2);
                await this.productRepository.update(dto.fourthProductId, { discountPrice: discountPrice });
            }
        }

        const fifthProduct = await this.productRepository.findOne(dto.fifthProductId);
        if (!fifthProduct) {
            throw new HttpException(
                "Fifth product not found",
                HttpStatus.NOT_FOUND
            );
        }
        else{
            const productPrice = fifthProduct.price;
            if (dto.fifthProductDiscount.includes('%')) {
                const fifthProductDiscount = dto.fifthProductDiscount.replace('%', '');
                const discount: any = (productPrice * (parseInt(fifthProductDiscount) / 100)).toFixed(2);
                const discountPrice: any = (productPrice - discount).toFixed(2);
                await this.productRepository.update(dto.fifthProductId, { discountPrice: discountPrice });
            }
            else {
                const discountPrice: any = (productPrice - parseInt(dto.fifthProductDiscount)).toFixed(2);
                await this.productRepository.update(dto.fifthProductId, { discountPrice: discountPrice });
            }
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<PromotionalPageCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
