import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateShippingAddressDto } from "./dto/create-shipping-address.dto";
import { UpdateShippingAddressDto } from "./dto/update-shipping-address.dto";
import { ShippingAddress } from "./entities/shipping-address.entity";

@EntityRepository(ShippingAddress)
export class ShippingAddressRepository extends Repository<ShippingAddress> {
    async createEntity(dto: any): Promise<ShippingAddress | Error> {
        try {
            const entity = this.create({
                name: dto.name,
                phone: dto.phone,
                email: dto.email,
                city: dto.city,
                postCode: dto.postCode,
                stateCode: dto.stateCode,
                country: dto.country,
                countryCode: dto.countryCode,
                addressOne: dto.addressOne,
                addressTwo: dto.addressTwo,
                userId: dto.userId,
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ShippingAddress | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateShippingAddressDto): Promise<ShippingAddress | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                phone: dto.phone,
                email: dto.email,
                city: dto.city,
                postCode: dto.postCode,
                stateCode: dto.stateCode,
                country: dto.country,
                countryCode: dto.countryCode,
                addressOne: dto.addressOne,
                addressTwo: dto.addressTwo,
                userId: dto.userId,
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ShippingAddress[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ShippingAddress)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByUserId(id: string): Promise<ShippingAddress | Error> {
        try {
            return await this.findOne({
                where: { userId: id, isDeleted: false }
            });
        }
        catch (e) {
            return e;   
        }
    }
}