import { Controller, Get, Post, Body, Patch, Param, Delete, Req, UseGuards } from '@nestjs/common';
import { AuthGuard } from 'src/core/guards/auth.guard';
import { Constants } from 'src/utils/constants';
import { CardInformationService } from './card-information.service';
import { CreateCardInformationDto } from './dto/create-card-information.dto';
import { UpdateCardInformationDto } from './dto/update-card-information.dto';

@Controller({
  path: 'card-information',
  version: Constants.API_VERSION_1  
})
export class CardInformationController {
  constructor(private readonly cardInformationService: CardInformationService) {}

  @Post()
  @UseGuards(AuthGuard)
  create(@Body() createCardInformationDto: CreateCardInformationDto, @Req() request: any) {
    return this.cardInformationService.create(createCardInformationDto, request);
  }

  @Get()
  @UseGuards(AuthGuard)
  findAll(@Req() request: any) {
    return this.cardInformationService.findAll(request);
  }

  @Get(':id')
  @UseGuards(AuthGuard)
  findOne(@Param('id') id: string) {
    return this.cardInformationService.findOne(id);
  }

  @Patch('/default/:id')
  @UseGuards(AuthGuard)
  makeDefault(@Param('id') id: string, @Req() request: any) {
    return this.cardInformationService.makeDefault(id, request);
  }

  @Patch(':id')
  @UseGuards(AuthGuard)
  update(@Param('id') id: string,
  @Body() updateCardInformationDto: UpdateCardInformationDto,
  @Req() request: any) {
    return this.cardInformationService.update(id, updateCardInformationDto, request);
  }

  @Delete(':id')
  @UseGuards(AuthGuard)
  remove(@Param('id') id: string) {
    return this.cardInformationService.remove(id);
  }
}
