import { Controller, Get, Post, Body, Patch, Param, Delete, UseGuards, Query, Req } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { AuthGuard } from 'src/core/guards/auth.guard';
import { Constants } from 'src/utils/constants';
import { CoreResourceService } from './core-resource.service';
import { CheckVerifyHashPasswordDto } from './dto/check-verify-hash-password-token';
import { CreateCoreResourceDto } from './dto/create-core-resource.dto';
import { GoogleLoginDto } from './dto/create-google-login.dto';
import { UpdateCoreResourceDto } from './dto/update-core-resource.dto';
import { UpdatePasswordUsingVerifyHashPasswordTokenDto } from './dto/update-password-using-verify-hash-password-token';

@Controller({
    path: 'core-resource',
    version: Constants.API_VERSION_1
})
export class CoreResourceController {
    constructor(private readonly coreResourceService: CoreResourceService) { }

    @Post('/google-login')
    doGoogleLogin(@Body() dto: GoogleLoginDto) {
        return this.coreResourceService.dogoogleLoginService(dto);
    }

    @Post('/verify-hash-token')
    async verifyHashToken(
        @Body()
        dto: CheckVerifyHashPasswordDto,
    ) {
        return await this.coreResourceService.verifyHashPasswordTokenRequest(
            dto,
        );
    }

    @Post('/hash-token/update-password')
    async updatePasswordUsingHashToken(
        @Body()
        dto: UpdatePasswordUsingVerifyHashPasswordTokenDto,
    ) {
        console.log(dto);
        return await this.coreResourceService.updatePasswordUsingTokenRequest(
            dto,
        );
    }

    @Get('/product-variants')
    productVariants() {
        return this.coreResourceService.productVariants();
    }

    @Get('/filter-sub-category')
    filterSubCategory(@Query("parentCategoryId") parentCategoryId?: string) {
        return this.coreResourceService.filterSubCategory(parentCategoryId);
    }

    @Get('/filter-child-category')
    filterChildCategory(@Query("subCategoryId") subCategoryId?: string) {
        return this.coreResourceService.filterChildCategory(subCategoryId);
    }

    @Get('/get-address')
    @UseGuards(AuthGuard)
    userAddress(@Req() request: any) {
        return this.coreResourceService.userAddress(request.user.id);
    }

    @Get('/get-contact-info')
    getContactInfo() {
        return this.coreResourceService.getContactInfo();
    }

    @Get('/header-footer')
    headerFooter() {
        return this.coreResourceService.headerFooter()
    }

    @Get('/home')
    home() {
        return this.coreResourceService.home();
    }

    @Get('/collection-page')
    collectionPage() {
        return this.coreResourceService.collectionPage();
    }

    @Get('/style-page')
    stylePage() {
        return this.coreResourceService.stylePage();
    }

    @Get('/learn-page')
    learnPage() {
        return this.coreResourceService.learnPage();
    }

    @Get('/gift-guide-menu')
    giftGuideMenu() {
        return this.coreResourceService.giftGuideMenu();
    }

    @Get('/glossary')
    glossary(
        @Query('alphabet') alphabet?: string
    ) {
        return this.coreResourceService.glossary(alphabet);
    }

    @Get('/gift-card')
    giftCard(
        @Query('page') page?: number,
        @Query('limit') limit?: number,
    ) {
        return this.coreResourceService.giftCard(page, limit);
    }

    @Get('/product-variant')
    productVariant() {
        return this.coreResourceService.productVariant();
    }

    @Get('/promotional-page')
    promotionalPage(@Query('page') page?: number, @Query('limit') limit?: number) {
        return this.coreResourceService.promotionalPage(page, limit);
    }

    @Get('/gallery-page')
    galleryPage() {
        return this.coreResourceService.galleryPage();
    }

    @Get('/about-page')
    aboutPage() {
        return this.coreResourceService.aboutPage();
    }

    @Get('/product-details/:id')
    productDetails(@Param('id') id: string) {
        return this.coreResourceService.productDetails(id);
    }

    @Get('/gift-card-details/:id')
    giftCardDetails(@Param('id') id: string) {
        return this.coreResourceService.giftCardDetails(id);
    }

}
