import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import CryptoUtils from 'src/utils/crypto.utils';
import { ResponseUtils } from 'src/utils/response.utils';
import { MoreThan } from 'typeorm';
import { BillingAddressRepository } from '../address/billing-address/billing-address.repository';
import { ShippingAddressRepository } from '../address/shipping-address/shipping-address.repository';
import { AuthService } from '../auth/auth.service';
import { LoginAuthDto } from '../auth/dto/login-auth.dto';
import { ChangePasswordTokenRepository } from '../change-password-token/change-password-token.repository';
import { AboutPageCmsRepository } from '../CMS/about-page-cms/about-page-cms.repository';
import { CollectionPageRepository } from '../CMS/collection-page-cms/collection-page-cms.repository';
import { ContactPageCmsRepository } from '../CMS/contact-page-cms/contact-page-cms.repository';
import { FooterCmsRepository } from '../CMS/footer-cms/footer-cms.repository';
import { GalleryPageCmsRepository } from '../CMS/gallery-page-cms/gallery-page-cms.repository';
import { GiftCardPageCmsRepository } from '../CMS/gift-card-page-cms/gift-card-page-cms.repository';
import { GiftGuideCmsRepository } from '../CMS/gift-guide-cms/gift-guide-cms.repository';
import { HomePageCmsRepository } from '../CMS/home-page-cms/home-page-cms.repository';
import { HomePageSectionFourCmsRepository } from '../CMS/home-page-section-four-cms/home-page-section-four-cms.repository';
import { HomePageSectionSevenCmsRepository } from '../CMS/home-page-section-seven-cms/home-page-section-seven-cms.repository';
import { HomePageSectionSixCmsRepository } from '../CMS/home-page-section-six-cms/home-page-section-six-cms.repository';
import { HomePageSectionThreeCmsRepository } from '../CMS/home-page-section-three-cms/home-page-section-three-cms.repository';
import { LearnPageCmsRepository } from '../CMS/learn-page-cms/learn-page-cms.repository';
import { ProductPageCmsRepository } from '../CMS/product-page-cms/product-page-cms.repository';
import { PromotionalPageCmsRepository } from '../CMS/promotional-page-cms/promotional-page-cms.repository';
import { StylePageSectionFiveCmsRepository } from '../CMS/style-page-section-five-cms/style-page-section-five-cms.repository';
import { StylePageSectionFourCmsRepository } from '../CMS/style-page-section-four-cms/style-page-section-four-cms.repository';
import { StylePageSectionOneCmsRepository } from '../CMS/style-page-section-one-cms/style-page-section-one-cms.repository';
import { StylePageSectionThreeCmsRepository } from '../CMS/style-page-section-three-cms/style-page-section-three-cms.repository';
import { AboutComponentCmsRepository } from '../component-cms/about-component-cms/about-component-cms.repository';
import { CollectionComponentCmsRepository } from '../component-cms/collection-component-cms/collection-component-cms.repository';
import { ContactComponentCmsRepository } from '../component-cms/contact-component-cms/contact-component-cms.repository';
import { GalleryComponentCmsRepository } from '../component-cms/gallery-component-cms/gallery-component-cms.repository';
import { GiftCardComponentCmsRepository } from '../component-cms/gift-card-component-cms/gift-card-component-cms.repository';
import { GiftCardDetailsComponentCmsRepository } from '../component-cms/gift-card-details-component-cms/gift-card-details-component-cms.repository';
import { HomeComponentCmsRepository } from '../component-cms/home-component-cms/home-component-cms.repository';
import { LearnComponentCmsRepository } from '../component-cms/learn-component-cms/learn-component-cms.repository';
import { ProductDetailsComponentCmsRepository } from '../component-cms/product-details-component-cms/product-details-component-cms.repository';
import { PromotionalComponentCmsRepository } from '../component-cms/promotional-component-cms/promotional-component-cms.repository';
import { ShopComponentCmsRepository } from '../component-cms/shop-component-cms/shop-component-cms.repository';
import { StyleComponentCmsRepository } from '../component-cms/style-component-cms/style-component-cms.repository';
import { ContactAddressRepository } from '../contact/contact-address/contact-address.repository';
import { GalleryRepository } from '../gallery/gallery.repository';
import { GiftCardRepository } from '../gift-card/gift-card.repository';
import { GlossaryRepository } from '../glossary/glossary.repository';
import { CustomerReviewRepository } from '../home/customer-review/customer-review.repository';
import { HomePageSectionTwoRepository } from '../home/home-page-section-two/home-page-section-two.repository';
import { HomeSliderRepository } from '../home/home-slider/home-slider.repository';
import { ProductChildCategoryRepository } from '../product/product-child-category/product-child-category.repository';
import { ProductColorRepository } from '../product/product-color/product-color.repository';
import { ProductHighlightRepository } from '../product/product-highlight/product-highlight.repository';
import { ProductMaterialRepository } from '../product/product-material/product-material.repository';
import { ProductParentCategory } from '../product/product-parent-category/entities/product-parent-category.entity';
import { ProductParentCategoryRepository } from '../product/product-parent-category/product-parent-category.repository';
import { ProductSizeRepository } from '../product/product-size/product-size.repository';
import { ProductSubCategoryRepository } from '../product/product-sub-category/product-sub-category.repository';
import { ProductTagRepository } from '../product/product-tag/product-tag.repository';
import { ProductsRepository } from '../product/product/product.repository';
import { SiteSocialLinksRepository } from '../site-social-links/site-social-links.repository';
import { CreateUserDto } from '../users/dto/create-user.dto';
import { UsersRepository } from '../users/user.repository';
import { CheckVerifyHashPasswordDto } from './dto/check-verify-hash-password-token';
import { GoogleLoginDto } from './dto/create-google-login.dto';
import { UpdatePasswordUsingVerifyHashPasswordTokenDto } from './dto/update-password-using-verify-hash-password-token';

@Injectable()
export class CoreResourceService {
    constructor(
        @InjectRepository(UsersRepository)
        private readonly userRepo: UsersRepository,
        private readonly authService: AuthService,
        @InjectRepository(ChangePasswordTokenRepository)
        private readonly cptRepo: ChangePasswordTokenRepository,
        @InjectRepository(ProductColorRepository)
        private readonly productColorRepo: ProductColorRepository,
        @InjectRepository(ProductSizeRepository)
        private readonly productSizeRepo: ProductSizeRepository,
        @InjectRepository(ProductTagRepository)
        private readonly productTagRepo: ProductTagRepository,
        @InjectRepository(ProductHighlightRepository)
        private readonly productHighlightRepo: ProductHighlightRepository,
        @InjectRepository(ProductMaterialRepository)
        private readonly productMaterialRepo: ProductMaterialRepository,
        @InjectRepository(ProductSubCategoryRepository)
        private readonly productSubCategoryRepo: ProductSubCategoryRepository,
        @InjectRepository(ProductChildCategoryRepository)
        private readonly productChildCategoryRepo: ProductChildCategoryRepository,
        @InjectRepository(ShippingAddressRepository)
        private readonly shippingAddressRepo: ShippingAddressRepository,
        @InjectRepository(BillingAddressRepository)
        private readonly billingAddressRepo: BillingAddressRepository,
        @InjectRepository(ContactAddressRepository)
        private readonly contactAddressRepo: ContactAddressRepository,
        @InjectRepository(FooterCmsRepository)
        private readonly footerCmsRepo: FooterCmsRepository,
        @InjectRepository(SiteSocialLinksRepository)
        private readonly siteSocialLinksRepo: SiteSocialLinksRepository,
        @InjectRepository(ProductsRepository)
        private readonly productRepo: ProductsRepository,
        @InjectRepository(HomePageCmsRepository)
        private readonly homePageCmsRepo: HomePageCmsRepository,
        @InjectRepository(HomeSliderRepository)
        private readonly homeSliderRepo: HomeSliderRepository,
        @InjectRepository(HomePageSectionTwoRepository)
        private readonly homePageSectionTwoRepo: HomePageSectionTwoRepository,
        @InjectRepository(HomePageSectionThreeCmsRepository)
        private readonly homePageSectionThreeCmsRepo: HomePageSectionThreeCmsRepository,
        @InjectRepository(HomePageSectionFourCmsRepository)
        private readonly homePageSectionFourCmsRepo: HomePageSectionFourCmsRepository,
        @InjectRepository(HomePageSectionSixCmsRepository)
        private readonly homePageSectionSixCmsRepo: HomePageSectionSixCmsRepository,
        @InjectRepository(HomePageSectionSevenCmsRepository)
        private readonly homePageSectionSevenCmsRepo: HomePageSectionSevenCmsRepository,
        @InjectRepository(CustomerReviewRepository)
        private readonly customerReviewRepo: CustomerReviewRepository,
        @InjectRepository(CollectionPageRepository)
        private readonly collectionPageRepo: CollectionPageRepository,
        @InjectRepository(ContactPageCmsRepository)
        private readonly contactPageCmsRepo: ContactPageCmsRepository,
        @InjectRepository(StylePageSectionOneCmsRepository)
        private readonly stylePageSectionOneCmsRepo: StylePageSectionOneCmsRepository,
        @InjectRepository(StylePageSectionThreeCmsRepository)
        private readonly stylePageSectionThreeCmsRepo: StylePageSectionThreeCmsRepository,
        @InjectRepository(StylePageSectionFourCmsRepository)
        private readonly stylePageSectionFourCmsRepo: StylePageSectionFourCmsRepository,
        @InjectRepository(StylePageSectionFiveCmsRepository)
        private readonly stylePageSectionFiveCmsRepo: StylePageSectionFiveCmsRepository,
        @InjectRepository(GalleryRepository)
        private readonly galleryRepo: GalleryRepository,
        @InjectRepository(GiftCardPageCmsRepository)
        private readonly giftCardPageCmsRepo: GiftCardPageCmsRepository,
        @InjectRepository(GiftCardRepository)
        private readonly giftCardRepo: GiftCardRepository,
        @InjectRepository(ProductPageCmsRepository)
        private readonly productPageCmsRepo: ProductPageCmsRepository,
        @InjectRepository(ProductParentCategoryRepository)
        private readonly productParentCatRepo: ProductParentCategoryRepository,
        @InjectRepository(LearnPageCmsRepository)
        private readonly learnPageCmsRepository: LearnPageCmsRepository,
        @InjectRepository(GiftGuideCmsRepository)
        private readonly giftGuideCmsRepository: GiftGuideCmsRepository,
        private readonly glossaryRepository: GlossaryRepository,
        @InjectRepository(PromotionalPageCmsRepository)
        private readonly promotionalPageCmsRepo: PromotionalPageCmsRepository,
        @InjectRepository(HomeComponentCmsRepository)
        private readonly homeComponentCmsRepo: HomeComponentCmsRepository,
        @InjectRepository(ShopComponentCmsRepository)
        private readonly shopComponentCmsRepo: ShopComponentCmsRepository,
        @InjectRepository(GalleryPageCmsRepository)
        private readonly galleryPageCmsRepo: GalleryPageCmsRepository,
        @InjectRepository(GalleryComponentCmsRepository)
        private readonly galleryComponentCmsRepo: GalleryComponentCmsRepository,
        @InjectRepository(StyleComponentCmsRepository)
        private readonly styleComponentCmsRepo: StyleComponentCmsRepository,
        @InjectRepository(AboutPageCmsRepository)
        private readonly aboutPageCmsRepo: AboutPageCmsRepository,
        @InjectRepository(AboutComponentCmsRepository)
        private readonly aboutComponentCmsRepo: AboutComponentCmsRepository,
        @InjectRepository(CollectionComponentCmsRepository)
        private readonly collectionComponentCmsRepo: CollectionComponentCmsRepository,
        @InjectRepository(LearnComponentCmsRepository)
        private readonly learnComponentCmsRepo: LearnComponentCmsRepository,
        @InjectRepository(ContactComponentCmsRepository)
        private readonly contactComponentCmsRepo: ContactComponentCmsRepository,
        @InjectRepository(PromotionalComponentCmsRepository)
        private readonly promotionalComponentCmsRepo: PromotionalComponentCmsRepository,
        @InjectRepository(GiftCardComponentCmsRepository)
        private readonly giftCardComponentCmsRepo: GiftCardComponentCmsRepository,
        @InjectRepository(GiftCardDetailsComponentCmsRepository)
        private readonly giftCardDetailsComponentCmsRepo: GiftCardDetailsComponentCmsRepository,
        @InjectRepository(ProductDetailsComponentCmsRepository)
        private readonly productDetailsComponentCmsRepo: ProductDetailsComponentCmsRepository,
    ) { }

    async dogoogleLoginService(dto: GoogleLoginDto) {

        let loginResponse = null;
        let checkUserEmail = await this.userRepo.findByEmailAddress(dto.email);
        if (!checkUserEmail) {
            let userCreateDto = {
                fullName: dto.name,
                email: dto.email,
                password: dto.token,
                phone: "",
                role: 3
            };
            const data = await this.userRepo.createEntity(userCreateDto as CreateUserDto);


            let dtoLogin = {
                email: dto.email,
                password: dto.token
            };
            loginResponse = await this.authService.login(dtoLogin as LoginAuthDto);

        }
        else {
            await this.userRepo.updateUserPassword(checkUserEmail['id'], await CryptoUtils.encrypt(dto.token));
            let dtoLogin = {
                email: dto.email,
                password: dto.token
            };
            loginResponse = await this.authService.login(dtoLogin as LoginAuthDto);
        }
        return loginResponse;
    }


    async verifyHashPasswordTokenRequest(
        dto: CheckVerifyHashPasswordDto,
    ) {

        //console.log(dto.hashPass);
        let userCheck = await this.cptRepo.findByHashToken(dto.hashPass);
        //console.log(userCheck);
        if (!userCheck) {
            return ResponseUtils.buildData(
                false,
            );
        }
        else {
            return ResponseUtils.buildData(
                true,
            );
        }



    }

    async updatePasswordUsingTokenRequest(
        dto: UpdatePasswordUsingVerifyHashPasswordTokenDto,
    ) {

        //console.log(dto.passToken);
        let userCheck = await this.cptRepo.findByHashToken(dto.passToken);
        //console.log('userCheck data = ', userCheck);
        if (!userCheck) {
            return ResponseUtils.buildData(
                false,
            );
        }
        else {
            //console.log('Email =', userCheck['email']);
            let userInfo = await this.userRepo.findByEmailAddress(userCheck['email']);
            if (!userInfo) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }

            //console.log(userInfo);

            //console.log('Updating Password');
            let updPass = await this.userRepo.updateUserPassword(userInfo['id'], await CryptoUtils.encrypt(dto.password));
            //console.log('updPass =', updPass);
            return ResponseUtils.buildData(
                true,
            );
        }
    }

    async productVariants() {
        const productColor = await this.productColorRepo.findByFilterQuery({});
        const productSize = await this.productSizeRepo.findByFilterQuery({});
        const productTag = await this.productTagRepo.findByFilterQuery({});
        const productHighlight = await this.productHighlightRepo.findByFilterQuery({});
        const productMaterial = await this.productMaterialRepo.findByFilterQuery({});

        return {
            productColor,
            productSize,
            productTag,
            productHighlight,
            productMaterial
        }
    }

    async filterSubCategory(parentCategoryId?: string): Promise<any | Error> {
        const data = await this.productSubCategoryRepo.findByFilterQuery({ parentCategoryId: parentCategoryId });

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product SubCategory Data Fetched", "data", data);
    }

    async filterChildCategory(subCategoryId?: string): Promise<any | Error> {
        const data = await this.productChildCategoryRepo.findByFilterQuery({ subCategoryId: subCategoryId });

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Product ChildCategory Data Fetched", "data", data);
    }

    async userAddress(id: string): Promise<any | Error> {
        const shippingAddress = await this.shippingAddressRepo.findByUserId(id);
        const billingAddress = await this.billingAddressRepo.findByUserId(id);

        const response = {
            shippingAddress: shippingAddress,
            billingAddress: billingAddress
        };

        return ResponseUtils.successResponseHandler(200, "User address data fetched!", "data", response);
    }

    async getContactInfo(): Promise<any | Error> {
        const contact = await this.contactAddressRepo.findByFilterQuery({});
        const contactPageCms = await this.contactPageCmsRepo.findAll();
        const contactComponetCms = await this.contactComponentCmsRepo.findAll();

        const response = {
            contact: contact ? contact : [],
            contactPageCms: contactPageCms ? contactPageCms : [],
            contactComponetCms: contactComponetCms ? contactComponetCms : []
        };

        return ResponseUtils.successResponseHandler(200, "Contact Info Data Fetched", "data", response);
    }

    async headerFooter(): Promise<any | Error> {
        const footerCms = await this.footerCmsRepo.findAll();
        const socialLink = await this.siteSocialLinksRepo.findByFilterQuery({});
        const topTwoProduct = await this.productRepo.topTwoProducts();

        const response = {
            footerCms: footerCms ? footerCms : [],
            socialLink: socialLink ? socialLink : [],
            topTwoProduct: topTwoProduct ? topTwoProduct : []
        }

        return ResponseUtils.successResponseHandler(200, "Header Footer Data Fetched", "data", response);
    }

    async home(): Promise<any | Error> {
        const homePageCms = await this.homePageCmsRepo.findAll();
        const homePageSlider = await this.homeSliderRepo.findByFilterQuery({});
        const homePageSectionTwo = await this.homePageSectionTwoRepo.findByFilterQuery({});
        const homePageSectionThree = await this.homePageSectionThreeCmsRepo.findAll();
        const homePageSectionFour = await this.homePageSectionFourCmsRepo.findAll();
        const homePageSectionSix = await this.homePageSectionSixCmsRepo.findAll();
        const homePageSectionSeven = await this.homePageSectionSevenCmsRepo.findAll();
        const customerReview = await this.customerReviewRepo.findByFilterQuery({});
        const gallery = await this.galleryRepo.findByFilterQuery({});
        const latestProduct = await this.productRepo.find({
            take: 10,
            order: { createdAt: 'DESC' },
            where: { isDeleted: false }
        });
        const HomeComponentCms = await this.homeComponentCmsRepo.findAll();

        const response = {
            homePageCms: homePageCms ? homePageCms : [],
            homePageSlider: homePageSlider ? homePageSlider : [],
            homePageSectionTwo: homePageSectionTwo ? homePageSectionTwo : [],
            homePageSectionThree: homePageSectionThree ? homePageSectionThree : [],
            homePageSectionFour: homePageSectionFour ? homePageSectionFour : [],
            homePageSectionSix: homePageSectionSix ? homePageSectionSix : [],
            homePageSectionSeven: homePageSectionSeven ? homePageSectionSeven : [],
            customerReview: customerReview ? customerReview : [],
            gallery: gallery ? gallery : [],
            latestProduct: latestProduct ? latestProduct : [],
            HomeComponentCms: HomeComponentCms ? HomeComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Home Page Data Fetched", "data", response);
    }

    async collectionPage(): Promise<any | Error> {
        const productTag = await this.productTagRepo.findByFilterQuery({});
        const collectionPage = await this.collectionPageRepo.findByFilterQuery({});
        const lastestTenProduct = await this.productRepo.find({
            take: 10,
            order: { createdAt: 'DESC' },
            where: { isDeleted: false }
        });
        const homePageSectionTwo = await this.homePageSectionTwoRepo.findByFilterQuery({});
        const collectionComponentCms = await this.collectionComponentCmsRepo.findAll();
        const productCategories = await this.productParentCatRepo.find({ 
            take: 6,
            order: { createdAt: 'DESC' },
            where: { isDeleted: false } 
        });

        const response = {
            productTag: productTag ? productTag : [],
            collectionPage: collectionPage ? collectionPage : [],
            lastestTenProduct: lastestTenProduct ? lastestTenProduct : [],
            homePageSectionTwo: homePageSectionTwo ? homePageSectionTwo : [],
            collectionComponentCms: collectionComponentCms ? collectionComponentCms : [],
            productCategories: productCategories ? productCategories : []
        }

        return ResponseUtils.successResponseHandler(200, "Collection Page Data Fetched", "data", response);
    }

    async stylePage(): Promise<any | Error> {
        const stylePageSectionOne = await this.stylePageSectionOneCmsRepo.findAll();
        const stylePageSectionThree = await this.stylePageSectionThreeCmsRepo.findAll();
        const stylePageSectionFour = await this.stylePageSectionFourCmsRepo.findAll();
        const stylePageSectionFive = await this.stylePageSectionFiveCmsRepo.findAll();
        const homePageSectionSix = await this.homePageSectionSixCmsRepo.findAll();
        const lastestTenProduct = await this.productRepo.find({
            take: 10,
            order: { createdAt: 'DESC' },
            where: { isDeleted: false }
        });
        const styleComponentCms = await this.styleComponentCmsRepo.findAll();

        const response = {
            stylePageSectionOne: stylePageSectionOne ? stylePageSectionOne : [],
            stylePageSectionThree: stylePageSectionThree ? stylePageSectionThree : [],
            stylePageSectionFour: stylePageSectionFour ? stylePageSectionFour : [],
            stylePageSectionFive: stylePageSectionFive ? stylePageSectionFive : [],
            homePageSectionSix: homePageSectionSix ? homePageSectionSix : [],
            lastestTenProduct: lastestTenProduct ? lastestTenProduct : [],
            styleComponentCms: styleComponentCms ? styleComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Style Page Data Fetched", "data", response);
    }

    async productDetails(id: string): Promise<any | Error> {
        let relaventProduct: any;
        const product = await this.productRepo.findOne({
            where: { id: id, isDeleted: false },
            relations: [
                'productParentCategory',
                'productsubCategory',
                'productChildCategory',
                'productImages',
                'productColorVariants',
                'productSizeVariants',
                'productTagVariants',
                'productMaterialVariants',
                'productHighlightVariants'
            ]
        });
        if (!product) {
            throw new HttpException(
                "Product not found!",
                HttpStatus.NOT_FOUND
            );
        }
        if (product) {
            const data = await this.productRepo.relaventProducts(product['subCategoryId']);
            relaventProduct = data.filter((item: any) => item.id !== product.id);
        }

        const productDetailsComponentCms = await this.productDetailsComponentCmsRepo.findAll();

        const response = {
            productDetails: product ? product : [],
            relaventProduct: relaventProduct ? relaventProduct : [],
            productDetailsComponentCms: productDetailsComponentCms ? productDetailsComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Product Details Data Fetched", "data", response);
    }

    async giftCard(page: number, limit: number): Promise<any | Error> {
        const giftCardCms = await this.giftCardPageCmsRepo.findAll();
        const count = await this.giftCardRepo.find({ where: { isDeleted: false } })
        const pageCount = Math.ceil(count.length / limit);
        const giftCard = await this.giftCardRepo.find({
            skip: (page - 1) * limit,
            take: limit,
            where: { isDeleted: false }
        });
        const giftCardComponentCms = await this.giftCardComponentCmsRepo.findAll();

        const response = {
            giftCardCms: giftCardCms ? giftCardCms : [],
            giftCard: giftCard ? giftCard : [],
            giftCardComponentCms: giftCardComponentCms ? giftCardComponentCms : [],
            page: page,
            limit: limit,
            pageCount: pageCount
        }

        return ResponseUtils.successResponseHandler(200, "Gift Card Data Fetched", "data", response);
    }

    async productVariant(): Promise<any | Error> {
        const productPageCms = await this.productPageCmsRepo.findAll();
        const productParentCategory = await this.productParentCatRepo.findByFilterQuery({});
        const productColor = await this.productColorRepo.findByFilterQuery({});
        const productSize = await this.productSizeRepo.findByFilterQuery({});
        const productTag = await this.productTagRepo.findByFilterQuery({});
        const productHighlight = await this.productHighlightRepo.findByFilterQuery({});
        const productMaterial = await this.productMaterialRepo.findByFilterQuery({});
        const shopComponentCms = await this.shopComponentCmsRepo.findAll();

        const response = {
            productPageCms: productPageCms ? productPageCms : [],
            productParentCategory: productParentCategory ? productParentCategory : [],
            productColor: productColor ? productColor : [],
            productSize: productSize ? productSize : [],
            productTag: productTag ? productTag : [],
            productHighlight: productHighlight ? productHighlight : [],
            productMaterial: productMaterial ? productMaterial : [],
            shopComponentCms: shopComponentCms ? shopComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Product Variant Data Fetched", "data", response);
    }

    async learnPage(): Promise<any | Error> {
        const pageCMS = await this.learnPageCmsRepository.findAll();
        const learnComponentCms = await this.learnComponentCmsRepo.findAll();

        const response = {
            pageCMS: pageCMS ? pageCMS : [],
            learnComponentCms: learnComponentCms ? learnComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Learn Page Data Fetched", "data", response);
    }

    async glossary(alphabet: string): Promise<any | Error> {
        const glossary = await this.glossaryRepository.find({ where: { category: alphabet, isDeleted: false } });
        const response = {
            glossary: glossary ? glossary : []
        }

        return ResponseUtils.successResponseHandler(200, "Glossary Data Fetched", "data", response);
    }

    async giftGuideMenu(): Promise<any | Error> {
        const giftGuideCMS = await this.giftGuideCmsRepository.findAll();
        const productParentCategory = await this.productParentCatRepo.findByFilterQuery({});

        const response = {
            giftGuideCMS: giftGuideCMS ? giftGuideCMS : [],
            productParentCategory: productParentCategory ? productParentCategory : []
        }

        return ResponseUtils.successResponseHandler(200, "Gift Guide Menu Page Data Fetched", "data", response);
    }

    async promotionalPage(page: number, limit: number): Promise<any | Error> {
        const promotionalPageCMS = await this.promotionalPageCmsRepo.findAll();

        const discountProduct = await this.productRepo.discountedProductWithColor(limit, page);
        const count = await this.productRepo.discountedProductWithColorCount();
        const pageCount = Math.ceil(count / limit);
        const productCategoryList = await this.productParentCatRepo.findByFilterQuery({});
        const promotionalComponentCms = await this.promotionalComponentCmsRepo.findAll();

        const response = {
            promotionalPageCMS: promotionalPageCMS ? promotionalPageCMS : [],
            discountProduct: discountProduct ? discountProduct : [],
            productCategoryList: productCategoryList ? productCategoryList : [],
            promotionalComponentCms: promotionalComponentCms ? promotionalComponentCms : [],
            page: page,
            limit: limit,
            pageCount: pageCount
        }

        return ResponseUtils.successResponseHandler(200, "Promotional Page Data Fetched", "data", response);
    }

    async galleryPage(): Promise<any | Error> {
        const galleryPageCMS = await this.galleryPageCmsRepo.findAll();
        const gallery = await this.galleryRepo.find({ where: { isDeleted: false } });
        const galleryComponentCMS = await this.galleryComponentCmsRepo.findAll();

        const response = {
            galleryPageCMS: galleryPageCMS ? galleryPageCMS : [],
            gallery: gallery ? gallery : [],
            galleryComponentCMS: galleryComponentCMS ? galleryComponentCMS : []
        }

        return ResponseUtils.successResponseHandler(200, "Gallery Page Data Fetched", "data", response);
    }

    async aboutPage(): Promise<any | Error> {
        const aboutPageCMS = await this.aboutPageCmsRepo.findAll();
        const aboutComponentCMS = await this.aboutComponentCmsRepo.findAll();

        const response = {
            aboutPageCMS: aboutPageCMS ? aboutPageCMS : [],
            aboutComponentCMS: aboutComponentCMS ? aboutComponentCMS : []
        }

        return ResponseUtils.successResponseHandler(200, "About Page Data Fetched", "data", response);
    }

    async giftCardDetails(giftCardId: string): Promise<any | Error> {
        const giftCardDetails = await this.giftCardRepo.findOne({ where: { id: giftCardId, isDeleted: false } });
        const giftCardDetailsComponentCms = await this.giftCardDetailsComponentCmsRepo.findAll();

        const response = {
            giftCardDetails: giftCardDetails ? giftCardDetails : [],
            giftCardDetailsComponentCms: giftCardDetailsComponentCms ? giftCardDetailsComponentCms : []
        }

        return ResponseUtils.successResponseHandler(200, "Gift Card Data Fetched", "data", response);
    }
}
