import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { GalleryService } from './gallery.service';
import { CreateGalleryDto } from './dto/create-gallery.dto';
import { UpdateGalleryDto } from './dto/update-gallery.dto';

@Controller({ path: 'gallery', version: Constants.API_VERSION_1 })

export class GalleryController {
    constructor(private readonly service: GalleryService) { }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'image', maxCount: 1 }
        ]
    ))
    create(
        @Body() dto: CreateGalleryDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'image', maxCount: 1 }
        ]
    ))
    update(
        @Param('id') id: string,
        @Body() dto: UpdateGalleryDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
