import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateHomePageSectionTwoDto } from "./dto/create-home-page-section-two.dto";
import { UpdateHomePageSectionTwoDto } from "./dto/update-home-page-section-two.dto";
import { HomePageSectionTwo } from "./entities/home-page-section-two.entity";

@EntityRepository(HomePageSectionTwo)
export class HomePageSectionTwoRepository extends Repository<HomePageSectionTwo> {
    async createEntity(dto: CreateHomePageSectionTwoDto): Promise<HomePageSectionTwo | Error> {
        try {
            const { image, title, description, buttonText, buttonLink } = dto;
            const entity = this.create({
                image,
                title,
                description,
                buttonText,
                buttonLink
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<HomePageSectionTwo | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateHomePageSectionTwoDto): Promise<HomePageSectionTwo | Error> {
        try {
            await this.update(id, {
                image : dto.image,
                title: dto.title,
                description: dto.description,
                buttonText : dto.buttonText,
                buttonLink : dto.buttonLink
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<HomePageSectionTwo[] | Error> {
        try {
            return await this.find({
                where: { 
                    ...query, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(HomePageSectionTwo)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}