import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateHomeSliderDto } from "./dto/create-home-slider.dto";
import { UpdateHomeSliderDto } from "./dto/update-home-slider.dto";
import { HomeSlider } from "./entities/home-slider.entity";

@EntityRepository(HomeSlider)
export class HomeSliderRepository extends Repository<HomeSlider> {
    async createEntity(dto: CreateHomeSliderDto): Promise<HomeSlider | Error> {
        try {
            const { image, buttonText, buttonLink } = dto;
            const entity = this.create({
                image,
                buttonText,
                buttonLink
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<HomeSlider | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateHomeSliderDto): Promise<HomeSlider | Error> {
        try {
            await this.update(id, {
                image : dto.image,
                buttonText : dto.buttonText,
                buttonLink : dto.buttonLink
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<HomeSlider[] | Error> {
        try {
            return await this.find({
                where: { 
                    ...query, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(HomeSlider)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}