import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateNecklaceSizeDto } from "./dto/create-necklace-size.dto";
import { UpdateNecklaceSizeDto } from "./dto/update-necklace-size.dto";
import { NecklaceSize } from "./entities/necklace-size.entity";

@EntityRepository(NecklaceSize)
export class NecklaceSizeRepository extends Repository<NecklaceSize> {
    async createEntity(dto: CreateNecklaceSizeDto): Promise<NecklaceSize | Error> {
        try {
            const { size, circumference } = dto;
            const entity = this.create({
                size,
                circumference
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<NecklaceSize | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateNecklaceSizeDto): Promise<NecklaceSize | Error> {
        try {
            await this.update(id, {
                size : dto.size,
                circumference : dto.circumference
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<NecklaceSize[] | Error> {
        try {
            return await this.find({
                order: {createdAt: "DESC"},
                where: { 
                    ...query, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(NecklaceSize)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}