import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { NecklaceSizeRepository } from './necklace-size.repository';
import { CreateNecklaceSizeDto } from './dto/create-necklace-size.dto';
import { UpdateNecklaceSizeDto } from './dto/update-necklace-size.dto';
import { NecklaceSize } from './entities/necklace-size.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class NecklaceSizeService {
    constructor(
        @InjectRepository(NecklaceSizeRepository)
        private readonly repository: NecklaceSizeRepository,
        private readonly spaceService: DoSpacesService
    ) { }

    async create(
        dto: CreateNecklaceSizeDto
    ): Promise<NecklaceSize | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateNecklaceSizeDto
    ): Promise<NecklaceSize | Error> {
        const res : any = await this.repository.findOneEntity(id);
        if(!res){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(): Promise<NecklaceSize[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<NecklaceSize[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
