import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateNewsletterSubscriberDto } from './dto/create-newsletter-subscriber.dto';
import { UpdateNewsletterSubscriberDto } from './dto/update-newsletter-subscriber.dto';
import { NewsletterSubscriber } from './entities/newsletter-subscriber.entity';
import { NewsletterSubscriberRepository } from './newsletter-subscriber.repository';

@Injectable()
export class NewsletterSubscriberService {
  constructor(
      @InjectRepository(NewsletterSubscriberRepository)
      private readonly repository: NewsletterSubscriberRepository,
  ) { }

  async create(dto: CreateNewsletterSubscriberDto): Promise<NewsletterSubscriber | Error> {
      const data = await this.repository.createEntity(dto);
      if (!data) {
          throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
          );
      }

      return ResponseUtils.successResponseHandler(201, "Data stored successfully!","data", data);
  }

  async findAll(): Promise<NewsletterSubscriber[] | Error> {
    
      const data = await this.repository.findByFilterQuery({});
      
      if (!data) {
          throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
          );
      }

      return ResponseUtils.successResponseHandler(200,"Newsletter subscriber data fetched","data", data);
  }

  async remove(id: string): Promise<boolean | Error> {
      const data = await this.repository.deleteEntity(id);
      if (!data) {
          throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
          );
      }

      return ResponseUtils.buildDeletedData(
              data,
              200,
              "Data deleted successfully!"
      );
  }
}
