import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { generateOrderId, generateRandomPassword } from 'src/helper/common-helper';
import { DirectPost } from 'src/helper/nmi-payment-gatway-helper';
import { CreateShippingAddressDto } from 'src/modules/address/shipping-address/dto/create-shipping-address.dto';
import { UpdateShippingAddressDto } from 'src/modules/address/shipping-address/dto/update-shipping-address.dto';
import { ShippingAddressRepository } from 'src/modules/address/shipping-address/shipping-address.repository';
import { CouponRepository } from 'src/modules/coupon/coupon.repository';
import { GiftCardRepository } from 'src/modules/gift-card/gift-card.repository';
import { CreateListingStatusDto } from 'src/modules/mail-sender/dto/create-listing-status.dto';
import { CreateMailSenderDto } from 'src/modules/mail-sender/dto/create-mail-sender.dto';
import { MailSenderService } from 'src/modules/mail-sender/mail-sender.service';
import { PaymentConfigurationRepository } from 'src/modules/payment-configuration/payment-configuration.repository';
import { PaymentGatewayRepository } from 'src/modules/payment-gateway/payment-gateway.repository';
import { ProductsRepository } from 'src/modules/product/product/product.repository';
import { UsersRepository } from 'src/modules/users/user.repository';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { GiftCardDetailsRepository } from '../gift-card-details/gift-card-details.repository';
import { OrderDetailsRepository } from '../order-details/order-details.repository';
import { CreateOrderByAdminDto } from './dto/create-order-admin.dto';
import { CreateOrderGuestDto } from './dto/create-order-guest.dto';
import { CreateOrderDto } from './dto/create-order.dto';
import { InvoicePaymentDto } from './dto/invoice-payment.dto';
import { UpdateOrderDto } from './dto/update-order.dto';
import { Order } from './entities/order.entity';
import { OrderRepository } from './order-repository';

@Injectable()
export class OrderService {
  constructor(
    @InjectRepository(UsersRepository)
    private readonly usersRepository: UsersRepository,
    @InjectRepository(ProductsRepository)
    private readonly productsRepository: ProductsRepository,
    @InjectRepository(OrderRepository)
    private readonly orderRepository: OrderRepository,
    private readonly servMail: MailSenderService,
    @InjectRepository(OrderDetailsRepository)
    private readonly orderDetailsRepository: OrderDetailsRepository,
    @InjectRepository(ShippingAddressRepository)
    private readonly shippingAddressRepository: ShippingAddressRepository,
    @InjectRepository(GiftCardRepository)
    private readonly giftCardRepository: GiftCardRepository,
    @InjectRepository(GiftCardDetailsRepository)
    private readonly giftCardDetailsRepository: GiftCardDetailsRepository,
    @InjectRepository(PaymentGatewayRepository)
    private readonly paymentGatewayRepository: PaymentGatewayRepository,
    @InjectRepository(PaymentConfigurationRepository)
    private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
    @InjectRepository(CouponRepository)
    private readonly couponRepository: CouponRepository,
  ) { }

  getItemInfo(products: any, productId: any) {
    let newProductId: any = productId;
    let newProductInfo: any = {};

    products.forEach((productInfo: any) => {
      if (productInfo && productInfo.id === newProductId) {
        newProductInfo = productInfo;
      }
    });
    //console.log(JSON.stringify(newProductInfo));
    return newProductInfo;
  };

  getGiftCardItemInfo(giftCards: any, giftCardId: any) {
    let newGiftCardId: any = giftCardId;
    let newGiftCardInfo: any = {};

    giftCards.forEach((giftCardInfo: any) => {
      if (giftCardInfo && giftCardInfo.id === newGiftCardId) {
        newGiftCardInfo = giftCardInfo;
      }
    });
    //console.log(JSON.stringify(newProductInfo));
    return newGiftCardInfo;
  };

  async createOrderByAdmin(dto: CreateOrderByAdminDto, user: any): Promise<Order | Error> {
    try {
      let productIds: any = [];
      let giftCardIds: any = [];
      let itemSubTotal: any = 0;
      let itemTotal: any = 0;
      let itemQuantity: any = 0;
      let itemPrice: any = 0;
      let productItems: any = [];
      let giftCardItems: any = [];
      let subTotal: any = 0;
      let total: any = 0;
      let totalQuantity: any = 0;
      let orderFor: any;

      orderFor = await this.usersRepository.findOne({
        where: { email: dto.email }
      });

      const orders = await this.orderRepository.findOrderIdEntity();
      const orderId = await generateOrderId(orders);


      if (!orderFor) {
        const generateRandomPass = generateRandomPassword(8);
        console.log("Random Password = ", generateRandomPass)
        const userDto = {
          email: dto.email,
          fullName: dto.fullName,
          displayName: null,
          phone: dto.phone,
          image: 'https://golden-gal.nyc3.cdn.digitaloceanspaces.com/default_cms.png',
          streetAddress: null,
          city: null,
          postCode: null,
          country: 'United states of America',
          userType: 'customer',
          role: 3,
          password: generateRandomPass
        };
        const createUser = await this.usersRepository.createEntity(userDto);
        orderFor = createUser;
        if (orderFor) {
          let mailDto = {
            "to": [
              dto.email
            ],
            "from": process.env.EMAIL_SENDER_MAIL,
            "subject": "Golden Gal account creation notification.",
            "username": dto.fullName,
            "password": generateRandomPass
          }
          await this.servMail.accocuntCreateCustomerNotify(mailDto as CreateMailSenderDto);
        }
        console.log("new user=", orderFor);
      }
      else if (orderFor && orderFor.isDeleted === true) {
        const data = await this.usersRepository.update(orderFor.id, {
          isDeleted: false
        });
      }

      // add new user billing address or update existing user billing address 
      if (orderFor) {
        const shippingAddress = dto.shippingAddress ? JSON.parse(dto.shippingAddress) : {};
        console.log("Shipping address = ", shippingAddress);
        if (shippingAddress) {
          const shippingDto = {
            name: shippingAddress.name,
            phone: shippingAddress.phone,
            email: shippingAddress.email,
            city: shippingAddress.city,
            postCode: shippingAddress.postCode,
            stateCode: shippingAddress.stateCode,
            country: shippingAddress.country,
            countryCode: shippingAddress.countryCode,
            addressOne: shippingAddress.addressOne,
            addressTwo: shippingAddress.addressTwo,
            userId: orderFor.id
          }
          const isUserExists = await this.shippingAddressRepository.findByUserId(orderFor.id);
          if (isUserExists) {
            let key = isUserExists['id'];
            const data = await this.shippingAddressRepository.updateEntity(key, shippingDto as UpdateShippingAddressDto);
            if (!data) {
              throw new HttpException(
                "Shipping address not updated",
                HttpStatus.BAD_REQUEST
              );
            }
            console.log("Shipping address updated = ", data);
          }
          else {
            const data = await this.shippingAddressRepository.createEntity(shippingDto as CreateShippingAddressDto);
            if (!data) {
              throw new HttpException(
                "Shipping address not created",
                HttpStatus.BAD_REQUEST
              );
            }
            console.log("Shipping address created = ", data);
          }
        }
      }

      // working with produts 
      dto.products.forEach(async (product: any) => {
        productIds.push(product.productId);
      });

      const products = await this.productsRepository.getProductsByQuery(productIds);

      await dto.products.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get products
        newItemInfo = this.getItemInfo(products, itemInfo.productId);

        //details of items object populate
        details = {
          productId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: orderFor.id,
          orderId: orderId,
          featuredImage: newItemInfo.featuredImage,
          variants: itemInfo.variants.toString(),
          quantity: itemInfo.quantity,
          createdBy: user.id
        };

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = itemInfo.price;
          itemSubTotal = itemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        productItems.push(details);
      });


      // working with gift cards
      dto.giftCard.forEach(async (giftCard: any) => {
        giftCardIds.push(giftCard.giftCardId);
      });

      const giftCard = await this.giftCardRepository.getGiftCardsByQuery(giftCardIds);
      //  loop through the gift cards
      await dto.giftCard.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get gift cards
        newItemInfo = this.getGiftCardItemInfo(giftCard, itemInfo.giftCardId);

        //details of items object populate
        details = {
          giftCardId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: user.id,
          orderId: orderId,
          image: newItemInfo.image,
          quantity: itemInfo.quantity,
          createdBy: user.id
        };

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.giftCardId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = itemInfo.price;
          itemSubTotal = itemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        giftCardItems.push(details);
      });

      // calculate discount 
      let discount: any = 0;
      if (dto.itemTotalDiscount !== "0") {
        if (dto.itemTotalDiscount.includes('%')) {
          const itemTotalDiscount = dto.itemTotalDiscount.replace('%', '');
          discount = (total * (parseInt(itemTotalDiscount) / 100)).toFixed(2);
        }
        else {
          discount = parseInt(dto.itemTotalDiscount);
        }
      }

      // product price total
      const productPriceTotal = parseFloat(total).toFixed(2);

      // calculate total
      total = discount > 0 ? (total - discount).toFixed(2) : total;

      const placeOrder = {
        orderId: orderId,
        customerId: orderFor.id,
        customerName: orderFor.fullName,
        productPriceTotal: productPriceTotal,
        totalQuantity: totalQuantity,
        itemTotalDiscount: dto.itemTotalDiscount,
        subTotal: dto.itemSubTotal ? dto.itemSubTotal : subTotal,
        total: dto.itemTotal ? dto.itemTotal : total,
        remarks: dto.remarks ? dto.remarks : null,
        shippingAddress: dto.shippingAddress,
        billingAddress: dto.billingAddress,
        paymentAmount: dto.amount ? dto.amount : 0,
        paymentMethodName: dto.paymentMethodName,
        paymentStatus: dto.paymentStatus,
        createdBy: user.id
      }
      const orderPlaced = await this.orderRepository.createEntity(placeOrder);
      console.log("order placed=>", orderPlaced);


      productItems.forEach(async (item: any) => {
        const detailsPlaced = await this.orderDetailsRepository.createEntity(item);
        console.log("products details placed=>", detailsPlaced);
      });

      giftCardItems.forEach(async (item: any) => {
        const detailsPlaced = await this.giftCardDetailsRepository.createEntity(item);
        console.log("gift card details placed=>", detailsPlaced);
      })

      if (orderPlaced) {
        if (dto.paymentMethodName === 'Credit/Debit Card') {
          const paymentConf = await this.paymentConfigurationRepository.find();
          if (!paymentConf) {
            throw new HttpException(
              "Payment security key not found",
              HttpStatus.NOT_FOUND
            );
          }
          const dp = new DirectPost(paymentConf[0].securityKey);
          const billingData = dto.billingAddress ? JSON.parse(dto.billingAddress) : {};
          const billingName = billingData.name;
          const billingInfo = {
            'first_name': billingName,
            'address1': billingData.addressOne,
            'city': billingData.city,
            'state': billingData.stateCode,
            'zip': billingData.postCode
          }
          dp.setBilling(billingInfo);
          const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);

          if (data['statusCode'] === 200 && data['transactionid']) {
            const payload = {
              customerId: orderFor.id,
              customerName: orderFor.fullName,
              invoiceID: orderId,
              paymentAmount: dto.amount,
              paymentDate: new Date(),
              paymentTransID: data['transactionid'],
              paymentPayload: JSON.stringify(data),
              transStatus: "SUCCESS"
            }
            const res = await this.paymentGatewayRepository.createEntity(payload);
            // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });

            let mailAdminDto = {
              to: [user.email],
              from: process.env.EMAIL_SENDER_MAIL,
              subject: 'Order Placed Notification.',
              listingOwner: user.fullName,
              listingStatus: "Pending",
              listingID: orderId
            };
            const adminMail = await this.servMail.sendNewListingNotification(
              mailAdminDto as CreateListingStatusDto,
            );
            return ResponseUtils.successResponseHandler(200, "Order Placed and Payment successful");
          }
          else {
            await this.orderRepository.hardDeleteEntity(orderPlaced.id, orderPlaced.orderId);
            return ResponseUtils.errorResponseHandler(400, "Payment failed! Please try again", 'errors', data);
          }
        }
        else {
          let mailAdminDto = {
            to: [user.email],
            from: process.env.EMAIL_SENDER_MAIL,
            subject: 'Order Placed Notification.',
            listingOwner: user.fullName,
            listingStatus: "Pending",
            listingID: orderId
          };
          const adminMail = await this.servMail.sendNewListingNotification(
            mailAdminDto as CreateListingStatusDto,
          );
          return ResponseUtils.successResponseHandler(200, "Order created successfully");
        }
      }
      else {
        return ResponseUtils.errorResponseHandler(400, "Failed to create order data!");
      }
    }
    catch (e) {
      console.log(e);
      return e;
    }
  }

  async createOrderByCustomer(dto: CreateOrderDto, user: any): Promise<Order | Error> {
    try {
      let productIds: any = [];
      let giftCardIds: any = [];
      let itemSubTotal: any = 0;
      let itemTotal: any = 0;
      let itemQuantity: any = 0;
      let itemPrice: any = 0;
      let productItems: any = [];
      let giftCardItems: any = [];
      let subTotal: any = 0;
      let total: any = 0;
      let totalQuantity: any = 0;

      const orders = await this.orderRepository.findOrderIdEntity();
      const orderId = await generateOrderId(orders);

      // add new user billing address or update existing user billing address 
      if (user) {
        const shippingAddress = dto.shippingAddress ? JSON.parse(dto.shippingAddress) : {};
        console.log("Shipping address = ", shippingAddress);
        if (shippingAddress) {
          const shippingDto = {
            name: shippingAddress.name,
            phone: shippingAddress.phone,
            email: shippingAddress.email,
            city: shippingAddress.city,
            postCode: shippingAddress.postCode,
            stateCode: shippingAddress.stateCode,
            country: shippingAddress.country,
            countryCode: shippingAddress.countryCode,
            addressOne: shippingAddress.addressOne,
            addressTwo: shippingAddress.addressTwo,
            userId: user.id
          }
          const isUserExists = await this.shippingAddressRepository.findByUserId(user.id);
          if (isUserExists) {
            let key = isUserExists['id'];
            const data = await this.shippingAddressRepository.updateEntity(key, shippingDto as UpdateShippingAddressDto);
            if (!data) {
              throw new HttpException(
                "Shipping address not updated",
                HttpStatus.BAD_REQUEST
              );
            }
          }
          else {
            const data = await this.shippingAddressRepository.createEntity(shippingDto as CreateShippingAddressDto);
            if (!data) {
              throw new HttpException(
                "Shipping address not created",
                HttpStatus.BAD_REQUEST
              );
            }
          }
        }
      }


      // working with produts 
      dto.products.forEach(async (product: any) => {
        productIds.push(product.productId);
      });

      const products = await this.productsRepository.getProductsByQuery(productIds);
      //  loop through the products
      await dto.products.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get products
        newItemInfo = this.getItemInfo(products, itemInfo.productId);

        //details of items object populate
        details = {
          productId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: user.id,
          orderId: orderId,
          featuredImage: newItemInfo.featuredImage,
          variants: itemInfo.variants.toString(),
          quantity: itemInfo.quantity,
          createdBy: user.id,
          isGift: itemInfo.isGift
        };

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = itemInfo.price;
          itemSubTotal = itemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        productItems.push(details);
      });


      // working with gift cards
      dto.giftCard.forEach(async (giftCard: any) => {
        giftCardIds.push(giftCard.giftCardId);
      });

      const giftCard = await this.giftCardRepository.getGiftCardsByQuery(giftCardIds);
      //  loop through the gift cards
      await dto.giftCard.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get products
        newItemInfo = this.getGiftCardItemInfo(giftCard, itemInfo.giftCardId);

        //details of items object populate
        let isGift = false;
        if(itemInfo.giftEmail.length > 2)
        {
            isGift = true;
        }
        details = {
          giftCardId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: user.id,
          orderId: orderId,
          image: newItemInfo.image,
          quantity: itemInfo.quantity,
          createdBy: user.id,
          isGift: isGift,
          giftEmail: itemInfo.giftEmail
        };

        console.log('Gift = ', details);

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.giftCardId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = newItemInfo.price;
          itemSubTotal = newItemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        giftCardItems.push(details);
      });

      let couponDiscount = 0;
      if (dto.coupon) {
        couponDiscount = await this.couponRepository.getCouponDiscountByCouponNumber(dto.coupon, total);
        if (!couponDiscount) {
          throw new HttpException(
            "Coupon is not valid!",
            HttpStatus.BAD_REQUEST
          );
        }
      }

      const placeOrder = {
        orderId: orderId,
        customerId: user.id,
        customerName: user.fullName,
        productPriceTotal: total,
        couponNumber: dto.coupon ? dto.coupon : null,
        couponDiscount: couponDiscount,
        itemTotalDiscount: couponDiscount,
        totalQuantity: totalQuantity,
        subTotal: dto.itemSubTotal ? dto.itemSubTotal : subTotal,
        total: dto.itemTotal ? dto.itemTotal : total - couponDiscount,
        remarks: dto.remarks ? dto.remarks : null,
        shippingAddress: dto.shippingAddress,
        billingAddress: dto.billingAddress,
        paymentAmount: dto.amount ? dto.amount : 0,
        paymentMethodName: dto.paymentMethodName,
        paymentStatus: dto.paymentStatus,
        createdBy: user.id
      }
      const orderPlaced = await this.orderRepository.createEntity(placeOrder);


      productItems.forEach(async (item: any) => {
        const detailsPlaced = await this.orderDetailsRepository.createEntity(item);
      })

      giftCardItems.forEach(async (item: any) => {
        const detailsPlaced = await this.giftCardDetailsRepository.createEntity(item);
      })

      if (orderPlaced) {
        if (dto.paymentMethodName === 'Credit/Debit Card') {
          const paymentConf = await this.paymentConfigurationRepository.find();
          if (!paymentConf) {
            throw new HttpException(
              "Payment security key not found",
              HttpStatus.NOT_FOUND
            );
          }
          const dp = new DirectPost(paymentConf[0].securityKey);
          const billingData = dto.billingAddress ? JSON.parse(dto.billingAddress) : {};
          const billingName = billingData.name;
          const billingInfo = {
            'first_name': billingName,
            'address1': billingData.addressOne,
            'city': billingData.city,
            'state': billingData.stateCode,
            'zip': billingData.postCode
          }
          dp.setBilling(billingInfo);
          const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);

          if (data['statusCode'] === 200 && data['transactionid']) {
            const payload = {
              customerId: user.id,
              customerName: user.fullName,
              invoiceID: orderId,
              paymentAmount: dto.amount,
              paymentDate: new Date(),
              paymentTransID: data['transactionid'],
              paymentPayload: JSON.stringify(data),
              transStatus: "SUCCESS",
            }
            const res = await this.paymentGatewayRepository.createEntity(payload);
            // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });

            let mailAdminDto = {
              to: [user.email],
              from: process.env.EMAIL_SENDER_MAIL,
              subject: 'Order Placed Notification.',
              listingOwner: user.fullName,
              listingStatus: "Pending",
              listingID: orderId
            };
            const adminMail = await this.servMail.sendNewListingNotification(
              mailAdminDto as CreateListingStatusDto,
            );
            return ResponseUtils.successResponseHandler(200, "Order Placed and Payment successful");
          }
          else {
            await this.orderRepository.hardDeleteEntity(orderPlaced.id, orderPlaced.orderId);
            return ResponseUtils.errorResponseHandler(400, "Payment failed! Please try again", 'errors', data);
          }
        }
        else {
          let mailAdminDto = {
            to: [user.email],
            from: process.env.EMAIL_SENDER_MAIL,
            subject: 'Order Placed Notification.',
            listingOwner: user.fullName,
            listingStatus: "Pending",
            listingID: orderId
          };
          const adminMail = await this.servMail.sendNewListingNotification(
            mailAdminDto as CreateListingStatusDto,
          );
          return ResponseUtils.successResponseHandler(200, "Order created successfully");
        }
      }
      else {
        return ResponseUtils.errorResponseHandler(400, "Failed to create order data!");
      }
    }
    catch (e) {
      console.log(e);
      return e;
    }
  }

  async findAll(
    take: number,
    page: number,
    paymentStatus?: string,
    deliveryStatus?: string,
    startDate?: Date,
    endDate?: Date,
    customerName?: string,
    orderId?: string
  ): Promise<Order[] | Error> {
    let query: any = {};
    if (paymentStatus) query.paymentStatus = paymentStatus;
    if (deliveryStatus) query.deliveryStatus = deliveryStatus;
    if (startDate) query.startDate = startDate;
    if (endDate) query.endDate = endDate;
    if (customerName) query.customerName = customerName;
    if (orderId) query.orderId = orderId;

    const data = await this.orderRepository.findAllEntities(take, page, query);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Orders data successfully fetched.", "data", data);
  }

  async findOne(
    id: string
  ): Promise<Order | Error> {
    const data = await this.orderRepository.findOneEntity(id);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    const userDetails = await this.usersRepository.findOneEntity(data['customerId']);
    data["userDetails"] = userDetails ? userDetails : {};

    // payment log
    if (data) {
      const paymentLog = await this.paymentGatewayRepository.findByFilterQuery({ invoiceID: data['orderId'] });
      if (Object.keys(paymentLog).length > 0) {
        data["paymentDate"] = paymentLog ? paymentLog[0].paymentDate : "";
      }
    }
    return ResponseUtils.successResponseHandler(200, "Order data successfully fetched", "Orders", data);
  }


  async remove(id: string): Promise<boolean | Error> {
    const deleteOrder = await this.orderRepository.deleteEntity(id);
    if (!deleteOrder) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }

    const deleteOrderDetails = await this.orderDetailsRepository.deleteEntity(id);

    if (deleteOrder && deleteOrderDetails) {
      return ResponseUtils.buildDeletedData(
        true,
      );
    } else {
      return ResponseUtils.buildDeletedData(
        false,
      );
    }
  }


  async update(id: string, dto: UpdateOrderDto): Promise<Order | Error> {
    try {
      const order = await this.orderRepository.findOne(id);
      if (!order) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }

      dto.paymentStatus = dto.paymentStatus ? dto.paymentStatus : order.paymentStatus;
      dto.deliveryStatus = dto.deliveryStatus ? dto.deliveryStatus : order.deliveryStatus;

      const data = await this.orderRepository.updateOrder(id, dto);
      if (!data) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }

      if (order["deliveryStatus"] != dto.deliveryStatus) {
        const customerEmail = await this.usersRepository.findOne(data["customerId"]);
        let mailOrderDto = {
          to: [customerEmail['email']],
          from: process.env.EMAIL_SENDER_MAIL,
          subject: 'Order Status Changed notification.',
          listingOwner: data["customerName"],
          listingStatus: data["deliveryStatus"],
          listingID: data["orderId"]
        };
        const userMail = await this.servMail.sendListingNotification(
          mailOrderDto as CreateListingStatusDto,
        );
        console.log(userMail);
      }
      return ResponseUtils.successResponseHandler(200, "Order data updated", "data", data);
    } catch (err) {
      console.log(err);
      throw err;
    }
  }

  async getOrdersByCustomer(request: any, limit: number, page: number): Promise<Order[] | Error> {
    const orders = await this.orderRepository.getOrdersByCustomerId(request.user.id, limit, page);
    if (!orders) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Orders data successfully fetched.", "data", orders);
  }

  async invoicePayment(dto: InvoicePaymentDto, user: any): Promise<any | Error> {
    const invoice: any = await this.orderRepository.findOne({ orderId: dto.orderId });
    if (!invoice) {
      throw new HttpException(
        "Invoice not found",
        HttpStatus.NOT_FOUND
      )
    };
    const paymentConf = await this.paymentConfigurationRepository.find();
    if (!paymentConf) {
      throw new HttpException(
        "Payment security key not found",
        HttpStatus.NOT_FOUND
      );
    }
    const dp = new DirectPost(paymentConf[0].securityKey);
    const billingData = invoice.billingAddress ? JSON.parse(invoice.billingAddress) : {};
    const billingName = billingData.name;
    const billingInfo = {
      'first_name': billingName,
      'address1': billingData.addressOne,
      'city': billingData.city,
      'state': billingData.stateCode,
      'zip': billingData.postCode
    }
    dp.setBilling(billingInfo);
    const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);



    if (data['statusCode'] === 200 && data['transactionid']) {
      let paymentAmount = 0;
      if (Object.keys(invoice).length > 0 && invoice.paymentAmount != 0) {
        const amount: any = invoice.paymentAmount ? invoice.paymentAmount : 0;
        paymentAmount = dto.amount + parseFloat(amount);
      }
      const payload = {
        customerId: invoice.customerId,
        customerName: invoice.customerName,
        invoiceID: dto.orderId,
        paymentAmount: dto.amount,
        paymentDate: new Date(),
        paymentTransID: data['transactionid'],
        paymentPayload: JSON.stringify(data),
        transStatus: "SUCCESS"
      }
      const res = await this.paymentGatewayRepository.createEntity(payload);
      const updateOrder = await this.orderRepository.update(invoice['id'], {
        paymentStatus: 'PAID',
        paymentMethodName: 'Credit/Debit Card',
        paymentAmount: paymentAmount === 0 ? dto.amount : paymentAmount,
      });

      return ResponseUtils.successResponseHandler(200, "Payment done successfully.", "data", res);
    }
    else {
      return ResponseUtils.errorResponseHandler(400, "Payment failed! Please try again", 'errors', data);
    }
  }

  async createOrderByGuest(dto: CreateOrderGuestDto): Promise<Order | Error> {
    try {
      let productIds: any = [];
      let giftCardIds: any = [];
      let itemSubTotal: any = 0;
      let itemTotal: any = 0;
      let itemQuantity: any = 0;
      let itemPrice: any = 0;
      let productItems: any = [];
      let giftCardItems: any = [];
      let subTotal: any = 0;
      let total: any = 0;
      let totalQuantity: any = 0;
      const guestId = '9999999999999';
      const guestName = 'Guest';

      const orders = await this.orderRepository.findOrderIdEntity();
      const orderId = await generateOrderId(orders);

      // working with produts 
      dto.products.forEach(async (product: any) => {
        productIds.push(product.productId);
      });

      const products = await this.productsRepository.getProductsByQuery(productIds);
      //  loop through the products
      await dto.products.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get products
        newItemInfo = this.getItemInfo(products, itemInfo.productId);

        //details of items object populate
        details = {
          productId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: guestId,
          orderId: orderId,
          featuredImage: newItemInfo.featuredImage,
          variants: itemInfo.variants.toString(),
          quantity: itemInfo.quantity,
          createdBy: guestId,
        };

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = newItemInfo.price;
          itemSubTotal = newItemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        productItems.push(details);
      });


      // working with gift cards
      dto.giftCard.forEach(async (giftCard: any) => {
        giftCardIds.push(giftCard.giftCardId);
      });

      const giftCard = await this.giftCardRepository.getGiftCardsByQuery(giftCardIds);
      //  loop through the gift cards
      await dto.giftCard.forEach(async (itemInfo: any) => {
        let details: any = {};
        let newItemInfo: any = {};

        //Get products
        newItemInfo = this.getGiftCardItemInfo(giftCard, itemInfo.giftCardId);

        //details of items object populate
        details = {
          giftCardId: newItemInfo.id,
          name: newItemInfo.name,
          customerId: guestId,
          orderId: orderId,
          image: newItemInfo.image,
          quantity: itemInfo.quantity,
          createdBy: guestId,
        };

        //price addding for both without variants and with variants
        if (itemInfo && itemInfo.giftCardId && itemInfo.quantity > 0) {
          itemQuantity = itemInfo.quantity;
          itemPrice = newItemInfo.price;
          itemSubTotal = newItemInfo.price * itemQuantity;
          itemTotal = (itemPrice * itemQuantity);

        }

        if (itemQuantity > 0) totalQuantity += itemQuantity;
        if (itemSubTotal > 0) subTotal += itemSubTotal;
        if (itemTotal > 0) total += itemTotal;

        details.itemSubTotal = parseFloat(itemSubTotal).toFixed(2);
        details.itemTotal = parseFloat(itemTotal).toFixed(2);
        details.itemPrice = parseFloat(itemPrice).toFixed(2);
        giftCardItems.push(details);
      });


      const placeOrder = {
        orderId: orderId,
        customerId: guestId,
        customerName: guestName,
        productPriceTotal: total,
        totalQuantity: totalQuantity,
        subTotal: subTotal,
        total:  total,
        remarks: dto.remarks ? dto.remarks : null,
        shippingAddress: dto.shippingAddress,
        billingAddress: dto.billingAddress,
        paymentAmount: dto.amount ? dto.amount : 0,
        paymentMethodName: dto.paymentMethodName,
        paymentStatus: dto.paymentStatus,
        createdBy: guestId
      }
      const orderPlaced = await this.orderRepository.createEntity(placeOrder);
      console.log("order placed=>", orderPlaced);


      productItems.forEach(async (item: any) => {
        const detailsPlaced = await this.orderDetailsRepository.createEntity(item);
        console.log("product details placed=>", detailsPlaced);
      })

      giftCardItems.forEach(async (item: any) => {
        const detailsPlaced = await this.giftCardDetailsRepository.createEntity(item);
        console.log("gift card details placed=>", detailsPlaced);
      })

      if (orderPlaced) {
        if (dto.paymentMethodName === 'Credit/Debit Card') {
          const paymentConf = await this.paymentConfigurationRepository.find();
          if (!paymentConf) {
            throw new HttpException(
              "Payment security key not found",
              HttpStatus.NOT_FOUND
            );
          }
          const dp = new DirectPost(paymentConf[0].securityKey);
          const billingData = dto.billingAddress ? JSON.parse(dto.billingAddress) : {};
          const billingName = billingData.name;
          const billingInfo = {
            'first_name': billingName,
            'address1': billingData.addressOne,
            'city': billingData.city,
            'state': billingData.stateCode,
            'zip': billingData.postCode
          }
          dp.setBilling(billingInfo);
          const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);

          if (data['statusCode'] === 200 && data['transactionid']) {
            const payload = {
              customerId: guestId,
              customerName: guestName,
              invoiceID: orderId,
              paymentAmount: dto.amount,
              paymentDate: new Date(),
              paymentTransID: data['transactionid'],
              paymentPayload: JSON.stringify(data),
              transStatus: "SUCCESS",
            }
            const res = await this.paymentGatewayRepository.createEntity(payload);
            // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });

            // shipping billing address parse
            const billingData = dto.billingAddress ? JSON.parse(dto.billingAddress) : {};
            const shippingData = dto.shippingAddress ? JSON.parse(dto.shippingAddress) : {};

            let mailAdminDto = {
              to: [billingData.email, shippingData.email],
              from: process.env.EMAIL_SENDER_MAIL,
              subject: 'Order Placed Notification.',
              listingOwner: billingData.name,
              listingStatus: "Pending",
              listingID: orderId
            };
            const adminMail = await this.servMail.sendNewListingNotification(
              mailAdminDto as CreateListingStatusDto,
            );
            return ResponseUtils.successResponseHandler(200, "Guest Order Placed and Payment Successful");
          }
          else {
            await this.orderRepository.hardDeleteEntity(orderPlaced.id, orderPlaced.orderId);
            return ResponseUtils.errorResponseHandler(400, "Payment failed! Please try again", 'errors', data);
          }
        }
        else {
          // shipping billing address parse
          const billingData = dto.billingAddress ? JSON.parse(dto.billingAddress) : {};
          const shippingData = dto.shippingAddress ? JSON.parse(dto.shippingAddress) : {};

          let mailAdminDto = {
            to: [billingData.email, shippingData.email],
            from: process.env.EMAIL_SENDER_MAIL,
            subject: 'Order Placed Notification.',
            listingOwner: billingData.name,
            listingStatus: "Pending",
            listingID: orderId
          };
          const adminMail = await this.servMail.sendNewListingNotification(
            mailAdminDto as CreateListingStatusDto,
          );
          return ResponseUtils.successResponseHandler(200, "Guest order created successfully");
        }
      }
      else {
        return ResponseUtils.errorResponseHandler(400, "Failed to create guest order data!");
      }
    }
    catch (e) {
      console.log(e);
      return e;
    }
  }
}
