import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { UpdatePaymentConfigurationDto } from './dto/update-payment-configuration.dto';
import { PaymentConfiguration } from './entities/payment-configuration.entity';
import { PaymentConfigurationRepository } from './payment-configuration.repository';

@Injectable()
export class PaymentConfigurationService {
    constructor(
        @InjectRepository(PaymentConfigurationRepository)
        private readonly repository: PaymentConfigurationRepository
    ) { }

    async updateOrCreateData(
        dto: UpdatePaymentConfigurationDto, 
    ): Promise<PaymentConfiguration | Error> {
        const isExists = await this.repository.find();
        if(Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if(!data) {
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if(!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
        }
    }

    async findAll(): Promise<PaymentConfiguration[] | Error> {
        const data = await this.repository.findAll();
        if(!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
