import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductParentCategoryRepository } from './product-parent-category.repository';
import { CreateProductParentCategoryDto } from './dto/create-product-parent-category.dto';
import { UpdateProductParentCategoryDto } from './dto/update-product-parent-category.dto';
import { ProductParentCategory } from './entities/product-parent-category.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class ProductParentCategoryService {
    constructor(
        @InjectRepository(ProductParentCategoryRepository)
        private readonly repository: ProductParentCategoryRepository,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async create(
        dto: CreateProductParentCategoryDto,
        files: {
            image: UploadedMulterFileI,
        }): Promise<ProductParentCategory | Error> {
        const res: any = await this.repository.findByName(dto.name);
        if (res.length > 0) {
            throw new HttpException(
                "Product category name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }

        // process image one
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], 'ProductParentCategory/');
            dto.image = image;
        }

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product category created successfully", "data", data);
    }

    async findAll(): Promise<ProductParentCategory[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product category fetched successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateProductParentCategoryDto,
        files: {
            image: UploadedMulterFileI,
        }
    ): Promise<ProductParentCategory | Error> {
        const productParentCategory: any = await this.repository.findOneEntity(id);
        if (!productParentCategory) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        // process image one
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], 'ProductParentCategory/');
            dto.image = image;
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product category updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductParentCategory[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product category fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product category deleted successfully");
    }
}
