import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductSizeRepository } from './product-size.repository';
import { CreateProductSizeDto } from './dto/create-product-size.dto';
import { UpdateProductSizeDto } from './dto/update-product-size.dto';
import { ProductSize } from './entities/product-size.entity';

@Injectable()
export class ProductSizeService {
    constructor(
        @InjectRepository(ProductSizeRepository)
        private readonly repository: ProductSizeRepository
    ) { }

    async create(dto: CreateProductSizeDto): Promise<ProductSize | Error> {
        const res: any = await this.repository.findByName(dto.name);
        if (res.length > 0) {
            throw new HttpException(
                "Product size name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product size created successfully", "data", data);
    }

    async findAll(): Promise<ProductSize[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductSizeDto): Promise<ProductSize | Error> {
        const productSize: any = await this.repository.findOneEntity(id);
        if (!productSize) {
            throw new HttpException(
                "Product size not found.",
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductSize[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product size fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product size deleted successfully");
    }
}
