import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductSubCategoryDto } from "./dto/create-product-sub-category.dto";
import { UpdateProductSubCategoryDto } from "./dto/update-product-sub-category.dto";
import { ProductSubCategory } from "./entities/product-sub-category.entity";

@EntityRepository(ProductSubCategory)
export class ProductSubCategoryRepository extends Repository<ProductSubCategory> {

    async createEntity(dto: CreateProductSubCategoryDto): Promise<ProductSubCategory | Error> {
        try {
            const { name, parentCategoryId, parentCategoryName } = dto;
            const entity = this.create({
                name,
                parentCategoryId,
                parentCategoryName
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductSubCategory[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductSubCategory | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductSubCategoryDto): Promise<ProductSubCategory | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                parentCategoryId: dto.parentCategoryId,
                parentCategoryName: dto.parentCategoryName
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductSubCategory)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductSubCategory[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findAllEntitiesSelect(query: any): Promise<ProductSubCategory[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }
}