import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductSubCategoryRepository } from './product-sub-category.repository';
import { CreateProductSubCategoryDto } from './dto/create-product-sub-category.dto';
import { UpdateProductSubCategoryDto } from './dto/update-product-sub-category.dto';
import { ProductSubCategory } from './entities/product-sub-category.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class ProductSubCategoryService {
    constructor(
        @InjectRepository(ProductSubCategoryRepository)
        private readonly repository: ProductSubCategoryRepository,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async create(
        dto: CreateProductSubCategoryDto): Promise<ProductSubCategory | Error> {
        const res: any = await this.repository.findByName(dto.name);
        if (res.length > 0) {
            throw new HttpException(
                "Product sub category name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product sub category created successfully", "data", data);
    }

    async findAll(): Promise<ProductSubCategory[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sub category fetched successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateProductSubCategoryDto
    ): Promise<ProductSubCategory | Error> {
        const productSubCategory: any = await this.repository.findOneEntity(id);
        if (!productSubCategory) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sub category updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductSubCategory[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product sub category fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product sub category deleted successfully");
    }
}
