import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductTagVariantsDto } from "./dto/create-product-tag-variants.dto";
import { UpdateProductTagVariantsDto } from "./dto/update-product-tag-variants.dto";
import { ProductTagVariants } from "./entities/product-tag-variants.entity";

@EntityRepository(ProductTagVariants)
export class ProductTagVariantsRepository extends Repository<ProductTagVariants> {

    async createEntity(dto: CreateProductTagVariantsDto): Promise<ProductTagVariants | Error> {
        try {
            const { productId, productName, tagId, tagName } = dto;
            const entity = this.create({
                productId,
                productName,
                tagId,
                tagName
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductTagVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductTagVariants | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductTagVariantsDto): Promise<ProductTagVariants | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductTagVariants)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductTagVariants[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteTagVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductTagVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertTagVariant(newTag: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductTagVariants)
                .values(newTag)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductTagValues(id: any, newTagEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deleteTagVariant(id);
            await this.bulkInsertTagVariant(newTagEntity)
        } catch (e) {
            return e;
        }
    }
}