import { IsDecimal } from "class-validator";
import { CoreEntity } from "src/core/core.entity";
import { Entity, Column, ManyToOne, JoinColumn, OneToMany } from "typeorm";
import { ProductChildCategory } from "../../product-child-category/entities/product-child-category.entity";
import { ProductColorVariants } from "../../product-color-variants/entities/product-color-variants.entity";
import { ProductColor } from "../../product-color/entities/product-color.entity";
import { ProductHighlightVariants } from "../../product-highlight-variants/entities/product-highlight-variants.entity";
import { ProductImageGallery } from "../../product-image-gallery/entities/product-image-gallery.entity";
import { ProductMaterialVariants } from "../../product-material-variants/entities/product-material-variants.entity";
import { ProductParentCategory } from "../../product-parent-category/entities/product-parent-category.entity";
import { ProductSizeVariants } from "../../product-size-variants/entities/product-size-variants.entity";
import { ProductSize } from "../../product-size/entities/product-size.entity";
import { ProductSubCategory } from "../../product-sub-category/entities/product-sub-category.entity";
import { ProductTagVariants } from "../../product-tag-variants/entities/product-tag-variants.entity";

@Entity('products')
export class Product extends CoreEntity {
    @Column({ default: null, type: "longtext" })
    name: string;

    @Column({ default: 'https://golden-gal.nyc3.cdn.digitaloceanspaces.com/default_cms.png', nullable: true })
    featuredImage: string;

    @Column({ default: null })
    sku: string;

    @Column({ default: null })
    rating: string;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable: true})
    price: number;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable:true })
    cost: number;

    @Column({type: "decimal", precision: 10, scale: 2, default: 0, nullable:true })
    discountPrice: number;

    @Column({ default: 1 })
    stock: number;

    @Column({ default: 0 })
    weight: string;

    @Column({ default: null })
    vendorName: string;

    @Column()
    parentCategoryId: string;

    @Column()
    parentCategoryName: string;

    @Column({ default: null, nullable: true })
    subCategoryId: string;

    @Column({ default: null, nullable: true })
    subCategoryName: string;

    @Column({ default: null, nullable: true })
    childCategoryId: string;

    @Column({ default: null, nullable: true })
    childCategoryName: string;

    @Column({ default: 0, nullable: true })
    isFeatured: number

    @Column({ default: null, type: "longtext" })
    shortDescription: string;

    @Column({ default: null, type: "longtext" })
    longDescription: string;

    @Column({ default: 0, nullable: true})
    isMerch: number;

    @Column({ default: 0, nullable: true})
    isGiftGuide: number;

    @Column({ default: 0, nullable: true})
    isReadyToShip: number;

    @Column({ default: 0, nullable: true })
    totalOrder: number;

    @ManyToOne(() => ProductParentCategory)
    @JoinColumn([{ name: 'productParentCategory', referencedColumnName: "id" }])
    productParentCategory: ProductParentCategory;

    @ManyToOne(() => ProductSubCategory)
    @JoinColumn([{ name: 'productSubCategory', referencedColumnName: "id" }])
    productsubCategory: ProductParentCategory;

    @ManyToOne(() => ProductChildCategory)
    @JoinColumn([{ name: 'productChildCategory', referencedColumnName: "id" }])
    productChildCategory: ProductParentCategory;

    @OneToMany(() => ProductImageGallery, productImageGallery => productImageGallery.product)
    productImages: ProductImageGallery[];

    @OneToMany(() => ProductColorVariants, colorVariants => colorVariants.product)
    productColorVariants: ProductColorVariants[];

    @OneToMany(() => ProductSizeVariants, sizeVariants => sizeVariants.product)
    productSizeVariants: ProductSizeVariants[];

    @OneToMany(() => ProductTagVariants, tagVariants => tagVariants.product)
    productTagVariants: ProductTagVariants[];

    @OneToMany(() => ProductMaterialVariants, materialVariants => materialVariants.product)
    productMaterialVariants: ProductMaterialVariants[];

    @OneToMany(() => ProductHighlightVariants, highlightVariants => highlightVariants.product)
    productHighlightVariants: ProductHighlightVariants[];
}
