import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateVendorDto } from "./dto/create-vendor.dto";
import { UpdateVendorDto } from "./dto/update-vendor.dto";
import { Vendor } from "./entities/vendor.entity";

@EntityRepository(Vendor)
export class VendorRepository extends Repository<Vendor> {
    async createEntity(dto: CreateVendorDto): Promise<Vendor | Error> {
        try {
            const { name, location, email, phone } = dto;
            const entity = this.create({
                name,
                location,
                email,
                phone
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<Vendor | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateVendorDto): Promise<Vendor | Error> {
        try {
            await this.update(id, {
                name : dto.name,
                location : dto.location,
                email : dto.email,
                phone : dto.phone
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<Vendor[] | Error> {
        try {
            return await this.find({
                where: { 
                    ...query, isDeleted: false 
                },
                order: { 
                    createdAt: "DESC" 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Vendor)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}